<?php

/* --------------------------------------------------------------
   ContentServiceFactory.inc.php 2019-04-16
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ContentServiceFactory
 *
 * This class represents a factory for content services
 *
 * @category   System
 * @package    Content
 */
class ContentServiceFactory implements ContentServiceFactoryInterface
{
    /**
     * CodeIgniter QueryBuilder
     *
     * @var CI_DB_query_builder
     */
    protected $queryBuilder;
    
    /**
     * Content repository
     *
     * @var ContentRepository
     */
    protected $repository;
    
    /**
     * Content write service
     *
     * @var ContentWriteService
     */
    protected $writeService;
    
    /**
     * @var ContentReadService
     */
    protected $readService;
    
    /**
     * Content writer
     *
     * @var ContentWriter
     */
    protected $writer;
    
    /**
     * @var ContentReader
     */
    protected $reader;
    
    /**
     * @var ContentDeleter
     */
    protected $deleter;
    
    
    /**
     * ContentServiceFactory constructor
     *
     * @param CI_DB_query_builder $queryBuilder CodeIgniter QueryBuilder instance
     */
    public function __construct(CI_DB_query_builder $queryBuilder)
    {
        $this->queryBuilder = $queryBuilder;
    }
    
    
    /**
     * Return a content write service
     *
     * @return ContentWriteService
     */
    public function createWriteService(): ContentWriteService
    {
        if ($this->writeService === null) {
            $this->writeService = MainFactory::create(ContentWriteService::class, $this->repository());
        }
        
        return $this->writeService;
    }
    
    
    /**
     * @return ContentReadService
     */
    public function createReadService(): ContentReadService
    {
        if ($this->readService === null) {
            
            $this->readService = MainFactory::create(ContentReadService::class, $this->repository());
        }
        
        return $this->readService;
    }
    
    
    /**
     * Return a content repository
     *
     * @return ContentRepository
     */
    protected function repository(): ContentRepository
    {
        if ($this->repository === null) {
            $this->repository = MainFactory::create(ContentRepository::class,
                                                    $this->writer(),
                                                    $this->reader(),
                                                    $this->deleter());
        }
        
        return $this->repository;
    }
    
    
    /**
     * Return a content writer
     *
     * @return ContentWriter
     */
    protected function writer(): ContentWriter
    {
        if ($this->writer === null) {
            $this->writer = MainFactory::create(ContentWriter::class, $this->queryBuilder);
        }
        
        return $this->writer;
    }
    
    
    /**
     * @return ContentReader
     */
    public function reader(): ContentReader
    {
        if ($this->reader === null) {
            $this->reader = MainFactory::create(ContentReader::class, $this->queryBuilder);
        }
        
        return $this->reader;
    }
    
    
    /**
     * Create content deleter.
     *
     * @return ContentDeleter
     */
    public function deleter(): ContentDeleter
    {
        if ($this->deleter === null) {
            $this->deleter = MainFactory::create(ContentDeleter::class, $this->queryBuilder);
        }
        
        return $this->deleter;
    }
}