<?php
/* --------------------------------------------------------------
   StyleEditController.inc.php 2019-03-14
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Api\Controllers;

use Gambio\StyleEdit\Api\StyleEditAuthenticator;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection;
use Gambio\StyleEdit\Core\Components\Theme\StyleEditThemeService;
use Gambio\StyleEdit\Core\Language;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\Core\TranslatedException;

/**
 * Class StyleEditController
 * @package Gambio\StyleEdit\Api\Controllers
 */
class StyleEditController
{
    public const COMPONENT_URI_INDEX = 1;
    public const THEME_ID_URI_INDEX  = 2;
    /**
     * @var string[]
     */
    protected $uri;
    
    /**
     * @var \Slim\Slim
     */
    protected $api;
    
    /**
     * @var string
     */
    protected $themeId = false;
    
    /**
     * @var \Gambio\StyleEdit\Core\Components\BasicController
     */
    protected $componentController;
    
    protected $body;
    
    
    /**
     * StyleEditController constructor.
     *
     * @param \Slim\Slim $api
     * @param array      $uri
     * @param            $language
     *
     * @throws \Gambio\StyleEdit\Core\TranslatedException
     * @throws \ReflectionException
     */
    public function __construct(\Slim\Slim $api, array $uri, $language)
    {
        $uri        = $this->cleanUri($uri);
        $languageId = SingletonPrototype::instance()
            ->get(\LanguageHelperInterface::class)
            ->getLanguageIdByCode(new \LanguageCode(new \StringType($language)));
        //Setup the language
        SingletonPrototype::instance()->setUp(Language::class, new Language($language, $languageId->asInt()));
        
        $this->api  = $api;
        $this->uri  = $uri;
        $this->body = file_get_contents("php://input");
        
        $this->initializeTheme();
        
        //Ex: StyleEdit/theme/HoneyGrid
        $controllerName = $this->getComponentControllerClassName($this->uri[self::COMPONENT_URI_INDEX]);
        SingletonPrototype::instance()->setUp(\Slim\Slim::class, $api);
        
        $this->componentController = SingletonPrototype::instance()->get($controllerName);
        if (!$this->componentController) {
            throw new TranslatedException('INVALID_COMPONENT', [$controllerName]);
        }
        
        if ($this->componentController->requiresAuthentication()) {
            (new StyleEditAuthenticator)->authorize();
        }
    }
    
    
    /**
     * @param $name
     *
     * @return string
     */
    protected function getComponentControllerClassName($name): string
    {
        //first - check inner components
        $classname         = str_replace('-', '', ucwords($name, '-'));
        $controllerName    = "{$classname}Controller";
        $internalClassName = "Gambio\\StyleEdit\\Core\\Components\\$classname\\$controllerName";
        if (class_exists($internalClassName)) {
            return $internalClassName;
        }
        
        return $controllerName;
    }
    
    
    /**
     *
     */
    private function initializeTheme()
    {
        //StyleEdit/COMPONENT_NAME
        $themeCfgLst = SingletonPrototype::instance()->get(ThemeConfigurationCollection::class);
        
        if (array_key_exists(self::THEME_ID_URI_INDEX, $this->uri)
            && $themeCfgLst->hasKey($this->uri[self::THEME_ID_URI_INDEX])) {
            $this->themeId = $this->uri[self::THEME_ID_URI_INDEX];
            SingletonPrototype::instance()->get(StyleEditThemeService::class)->initialize($this->themeId);
        }
    }
    
    
    /**
     * @param array $uri
     *
     * @return array
     */
    private function cleanUri(array $uri)
    {
        while (count($uri) > 0 && (!trim($uri[count($uri) - 1]))) {
            array_pop($uri);
        }
        
        return $uri;
    }
    
    
    /**
     * @return mixed
     * @throws \Exception
     */
    public function get()
    {
        $this->componentController->get($this->uri);
    }
    
    
    /**
     * @return mixed
     */
    public function delete()
    {
        return $this->componentController->delete($this->uri, $this->body);
    }
    
    
    /**
     * @return mixed
     */
    public function patch()
    {
        return $this->componentController->patch($this->uri, $this->body);
    }
    
    
    /**
     * @return mixed
     */
    public function put()
    {
        return $this->componentController->put($this->uri, $this->body);
    }
    
    
    /**
     * @throws \Exception
     */
    public function post()
    {
        return $this->componentController->post($this->uri, $this->body);
    }
    
}