<?php
/*--------------------------------------------------------------------------------------------------
    ContentZoneService.php 2019-10-14
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2016 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

namespace Gambio\StyleEdit\Core\Components\ContentZone\Repositories;

use FileNotFoundException;
use FilesystemAdapter;
use Gambio\StyleEdit\Core\Components\ContentZone\Entities\ContentZoneData;
use Gambio\StyleEdit\Core\Components\ContentZone\Entities\ContentZoneOption;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Language;
use ReflectionException;

/**
 * Class ContentZoneRepository
 * @package Gambio\StyleEdit\Core\Components\ContentZone\Repositories
 */
class ContentZoneRepository
{
    /**
     * @var string
     */
    protected const JSON_FILE_PATTERN = '/\.json$/i';
    protected const JSON_DEFAULT_PATTERN = '/\.default\.json$/i';
    /**
     * @var ThemeConfiguration
     */
    protected $currentTheme;
    /**
     * @var FilesystemAdapter
     */
    protected $filesystem;


    /**
     * ContentZoneRepository constructor.
     *
     * @param CurrentThemeInterface $currentTheme
     * @param FilesystemAdapter $filesystem
     */
    public function __construct(CurrentThemeInterface $currentTheme, FilesystemAdapter $filesystem)
    {
        $this->currentTheme = $currentTheme;
        $this->filesystem = $filesystem;
    }


    /**
     * @param ContentZoneOption $option
     * @param Language $languageCode
     *
     * @throws FileNotFoundException
     */
    public function saveContentZoneTemplate(ContentZoneOption $option, Language $languageCode): void
    {
        $fileContent = $option->content()->htmlContent($languageCode);

        $filePath = $this->currentTheme->id() . '/html/system/content_zone_' . $option->id() . '_'
            . $languageCode->code() . '.html';
        
        if ($this->filesystem->has($filePath)) {
            $this->filesystem->delete($filePath);
        }

        $this->filesystem->write($filePath, $fileContent);
    }


    /**
     * @param ContentZoneOption $option
     *
     * @throws FileNotFoundException
     */
    public function saveContentZoneData(ContentZoneOption $option): void
    {
        $option->content()->persist();

        $filePath = $this->currentTheme->id() . '/contentzones/' . $option->id() . '.json';
        
        if ($this->filesystem->has($filePath)) {
            $this->filesystem->delete($filePath);
        }

        $this->filesystem->write($filePath, json_encode($option, JSON_PRETTY_PRINT));
    }


    /**
     * @return ContentZoneData
     * @throws FileNotFoundException
     * @throws ReflectionException
     */
    public function getContentZoneData(): ContentZoneData
    {
        $contentZoneJsonDirectory = $this->currentTheme->id() . '/contentzones/';
        
        $contentZones = [];

        $defaultFile = [];
        $customFiles = [];

        foreach ($this->filesystem->listContents($contentZoneJsonDirectory) as $file) {

            if (preg_match(self::JSON_DEFAULT_PATTERN, $file['basename'])) {
                $contentZoneName = preg_replace(self::JSON_DEFAULT_PATTERN, '', $file['basename']);
                $defaultFile[$contentZoneName] = $file['path'];
            } elseif (preg_match(self::JSON_FILE_PATTERN, $file['basename'])) {
                $contentZoneName = preg_replace(self::JSON_FILE_PATTERN, '', $file['basename']);
                $customFiles[$contentZoneName] = $file['path'];
            }
        }

        $contentZoneFiles = array_merge($defaultFile, $customFiles);
        foreach ($contentZoneFiles as $contentZoneFile) {
            $jsonContent = $this->filesystem->read($contentZoneFile);
            $contentZones[] = json_decode($jsonContent, false);
        }


        return new ContentZoneData($contentZones);
    }
}
