<?php
/* --------------------------------------------------------------
   ThemeService.php 2019-09-27
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Core\Components\Theme;

use Exception;
use ExistingDirectory;
use Gambio\GX\Services\System\ThemeSettings\ThemeSettingsService;
use Gambio\StyleEdit\Core\Command\TransactionalCommandInvoker;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection;
use Gambio\StyleEdit\Core\Components\Theme\Repositories\PublishedThemeRepository;
use Gambio\StyleEdit\Core\Components\Theme\Repositories\StyleEditThemeRepository;
use Gambio\StyleEdit\Core\Components\Theme\Repositories\ThemeConfigurationRepository;
use Gambio\StyleEdit\Core\Language;
use Gambio\StyleEdit\Core\Options\Commands\ClearCacheCommand;
use Gambio\StyleEdit\Core\Options\Entities\AbstractComponentGroupOption;
use Gambio\StyleEdit\Core\Options\Entities\AbstractOption;
use Gambio\StyleEdit\Core\Repositories\ConfigurationRepository;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\Core\TranslatedException;
use Gambio\StyleEdit\StyleEditConfiguration;
use MainFactory;
use StyleEditServiceFactory;

/**
 * Class ThemeService
 * @package Gambio\StyleEdit\Core\Components\Theme
 */
class StyleEditThemeService
{
    
    /**
     * @var StyleEditThemeRepository
     */
    protected $repository;
    
    /**
     * @var ThemeConfigurationRepository
     */
    protected $configurationRepository;
    
    /**
     * @var PublishedThemeRepository
     */
    protected $publishedThemeRepository;
    
    /**
     * @var StyleEditConfiguration
     */
    protected $configuration;
    
    
    /**
     * ThemeService constructor.
     *
     * @param StyleEditConfiguration $configuration
     */
    public function __construct(StyleEditConfiguration $configuration)
    {
        $this->configuration = $configuration;
    }
    
    
    /**
     * @param string $themeId
     *
     * @return ThemeConfiguration
     * @throws TranslatedException
     * @throws \ReflectionException
     */
    public function getConfigurationById($themeId): ThemeConfiguration
    {
        return $this->configurationRepository()->getById($themeId);
    }
    
    
    /**
     * @return ThemeConfigurationRepository
     * @throws Exception
     */
    protected function configurationRepository(): ThemeConfigurationRepository
    {
        if ($this->configurationRepository === null) {
            $this->configurationRepository = ThemeConfigurationRepository::create();
            if (!$this->configurationRepository) {
                throw new Exception('ConfigurationRepository was not initialized');
            }
        }
        
        return $this->configurationRepository;
    }
    
    
    /**
     * @param ThemeConfiguration                                               $parentTheme
     * @param                                                                  $data
     *
     * @return ThemeConfiguration
     * @throws Exception
     */
    public function duplicateTheme(ThemeConfiguration $parentTheme, $data): ThemeConfiguration
    {
        $data     = get_object_vars($data);
        $newTitle = $data['title'] ?? $this->createTitleForDuplicateTheme($parentTheme->title());
        $newId    = $data['id'] ?? $parentTheme->id() . date('Y-m-d_H-i-s');
        
        $newTheme = ThemeConfiguration::cloneThemeConfiguration($parentTheme, (object)$data, $newId, $newTitle);
        
        return $this->repository()->copyTo($parentTheme, $newTheme);
    }
    
    
    /**
     * @param $name
     *
     * @return string
     * @throws \ReflectionException
     */
    protected function createTitleForDuplicateTheme($name): string
    {
        $language = SingletonPrototype::instance()->get(Language::class);
        
        switch ($language->code()) {
            case 'de' :
                
                return $name . ' - Kopie';
            
            case 'en' :
            default   :
                
                return $name . ' - Copy';
        }
    }
    
    
    /**
     * @return StyleEditThemeRepository
     * @throws Exception
     */
    protected function repository(): StyleEditThemeRepository
    {
        if ($this->repository === null) {
            $this->repository = SingletonPrototype::instance()->get(StyleEditThemeRepository::class);
            if (!$this->repository) {
                throw new Exception('ThemeRepository was not initialized');
            }
        }
        
        return $this->repository;
    }
    
    
    /**
     * @param bool $alwaysClearPublicTheme
     *
     * @throws Exception
     */
    public function clearPublishedTheme(bool $alwaysClearPublicTheme = false): void
    {
        $dir = StyleEditServiceFactory::service()->getPublishedThemePath();
        if ($dir) {
            $directory = new ExistingDirectory($this->configuration->baseFolderPath() . $dir);
            $this->publishedThemeRepository()->clearPublishedTheme($directory);
            $cacheControl = SingletonPrototype::instance()->get(\CacheControl::class);
            $cacheControl->clear_content_view_cache();
            $cacheControl->clear_templates_c();
        }
        if ($alwaysClearPublicTheme && !in_array($dir, ['public/theme', 'public/theme/'])) {
            $directory = new ExistingDirectory($this->configuration->baseFolderPath() . 'public/theme');
            $this->publishedThemeRepository()->clearPublishedTheme($directory);
        }
    }
    
    
    /**
     * @return bool|PublishedThemeRepository|mixed
     * @throws Exception
     */
    public function publishedThemeRepository()
    {
        if ($this->publishedThemeRepository === null) {
            $this->publishedThemeRepository = SingletonPrototype::instance()->get(PublishedThemeRepository::class);
            if (!$this->publishedThemeRepository()) {
                throw new Exception('PublishedThemeRepository was not initialized');
            }
        }
        
        return $this->publishedThemeRepository;
    }
    
    
    /**
     * @return string
     * @throws Exception
     */
    public function createPreviewFolder(): string
    {
        $this->publishedThemeRepository()->removeOldPreviewThemes($this->configuration->publicFolderPath() . 'tmp/', 5);
        
        return $this->publishedThemeRepository()->createPreviewFolder();
    }
    
    
    /**
     * Delete Theme.
     *
     * @param ThemeConfiguration $theme
     *
     * @throws Exception
     */
    public function deleteTheme(ThemeConfiguration $theme)
    {
        if (!$theme->removable()) {
            throw new Exception('The theme is a undeletable Gambio theme');
        }
        
        if (count($theme->children()) > 0) {
            throw new Exception('Theme undeletable has children');
        }
        
        $this->repository()->delete($theme);
    }
    
    
    /**
     * @param string $themeId
     *
     * @throws Exception
     */
    public function initialize($themeId)
    {
        //search for the available template list
        $theme = $this->configurationRepository()->get()->get($themeId);
        SingletonPrototype::instance()->setUp(CurrentThemeInterface::class, $theme);
        //load the selected template configuration
        $this->loadThemeOverloadClasses($theme);
    }
    
    
    /**
     * @param $themeConfig
     */
    public function loadThemeOverloadClasses(ThemeConfiguration $themeConfig)
    {
        $setupComponentScript = $themeConfig->path() . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR
                                . 'setup.php';
        if (file_exists($setupComponentScript)) {
            require_once $setupComponentScript;
        }
    }
    
    
    /**
     * @param $data
     *
     * @throws TranslatedException
     */
    public function save($data)
    {
        $this->repository()->save($data);
        
        if (isset($data->options)) {
            $this->saveSettings($data->options);
        }
    }
    
    
    /**
     * @param $options
     *
     * @throws Exception
     */
    private function saveSettings($options): void
    {
        
        $commandList = [];
        $configurationRepository = SingletonPrototype::instance()->get(ConfigurationRepository::class);
        $clearCache = false;
        foreach ($options as $id => $jsonOption) {
            if (!isset($jsonOption->id)) {
                $jsonOption->id = $id;
            }
            
            
            $optionEntity = AbstractOption::createFromJsonObject($jsonOption);
            $clearCache = $clearCache || $optionEntity->requiresReload();
            
            $saveCommandName = ucfirst(str_replace('-', '', ucwords($optionEntity->type(), '-'))) . 'SaveCommand';
            $saveCommand     = SingletonPrototype::instance()->get($saveCommandName);
            
            if (!$saveCommand) {
                $saveCommandName = $optionEntity instanceof
                                   AbstractComponentGroupOption ? 'GroupOptionSaveCommand' : 'OptionSaveCommand';
                
                $saveCommand = SingletonPrototype::instance()->get($saveCommandName);
                $saveCommand->setConfigurationRepository($configurationRepository);
            }
            
            if (!$saveCommand) {
                throw new Exception('OptionSaveCommand was not initialized yet');
            }
            
            $saveCommand->setOption($optionEntity);
            $commandList[] = $saveCommand;
        }

        if($clearCache) {
            $commandList[] = SingletonPrototype::instance()->get(ClearCacheCommand::class);
        }
        $invoker = SingletonPrototype::instance()->get(TransactionalCommandInvoker::class);
        
        $invoker->runInsideTransaction($commandList);
    }
    
    
    /**
     * @param ThemeConfiguration $themeConfiguration
     * @param                    $data
     *
     * @return mixed
     * @throws Exception
     */
    public function patch(ThemeConfiguration $themeConfiguration, $data)
    {
        if (isset($data->active) && $data->active) {
            
            $themeSettingsService = SingletonPrototype::instance()->get(ThemeSettingsService::class);
            /** @var ThemeSettingsService $themeSettingsService */
            $themeSettingsService->activateTheme($themeConfiguration->id());
            unset($data->active);
            
            if (count((array)$data) === 0) {
                
                return null;
            }
        }
        
        $options = $data->options;
        
        unset($data->options);
        $result = $this->configurationRepository()->patch($themeConfiguration, $data);
        
        if (isset($options)) {
            $this->saveSettings($options);
        }
        
        return $result;
    }
    
    
    /**
     * @throws Exception
     */
    public function deactivateAll()
    {
        foreach ($this->configurationRepository()->get()->elements() as $themeConfig) {
            $this->configurationRepository()->deactivateTheme($themeConfig);
        }
    }
    
    
    /**
     *
     */
    public function __clone()
    {
        $this->configuration = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
    }
    
    
    /**
     * @throws Exception
     */
    public function updateThemeList()
    {
        $list = $this->configurationRepository()->get();
        SingletonPrototype::instance()->setUp(ThemeConfigurationCollection::class, $list);
    }
    
    
    /**
     * @param array $themes
     *
     * @return array
     */
    public function sortByActive(array $themes): array
    {
        usort($themes,
            function ($a, $b) {
                $aIsActive = !empty($a['active']);
                $bIsActive = !empty($b['active']);
                
                if (!$aIsActive && $bIsActive) {
                    return 1;
                } elseif ($aIsActive && !$bIsActive) {
                    return -1;
                }
                
                return 0;
            });
        
        return $themes;
    }
}