<?php
/* --------------------------------------------------------------
   SettingsController.php 2019-06-06
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Core\Options\Entities;

use Exception;
use Gambio\StyleEdit\Core\Components\Variant\Services\VariantService;
use Gambio\StyleEdit\Core\Repositories\ConfigurationRepository;
use Gambio\StyleEdit\Core\SingletonPrototype;
use stdClass;

/**
 * Class Category
 * @package Gambio\StyleEdit\Core\Components\Entities
 */
class ConfigurationCategory extends AbstractConfigurationType
{
    
    
    /**
     * @var \Gambio\StyleEdit\Core\Components\Entities\ConfigurationCategoryCollection
     */
    protected $categories;
    
    /**
     * @var \Gambio\StyleEdit\Core\Components\Entities\FieldSetCollection
     */
    protected $fieldSets;
    /**
     * @var VariantService
     */
    protected $variantService;
    
    protected $configurationRepository;
    
    /**
     * Categories which are only active for
     * a specific variant have this property
     *
     * @var string
     */
    protected $for;
    
    
    /**
     * @param                              $jsonCategory
     * @param                              $variantService
     * @param ConfigurationRepository|null $configurationRepository
     *
     * @return ConfigurationCategory
     * @throws \ReflectionException
     * @throws Exception
     */
    public static function createFromJsonObject(
        $jsonCategory,
        VariantService $variantService,
        ConfigurationRepository $configurationRepository = null
    ): ConfigurationCategory {
        $result = SingletonPrototype::instance()->get(static::class);
        
        if (!$result instanceof self) {
            throw new Exception('Category instance must inherit ' . self::class . '!');
        }
        $result->variantService          = $variantService;
        $result->configurationRepository = $configurationRepository;
        $result->initializeFromJsonObject($jsonCategory, $configurationRepository);
        
        return $result;
    }
    
    
    /**
     * @param                              $jsonObject
     *
     * @param VariantService               $variantService
     * @param ConfigurationRepository|null $configurationRepository
     *
     * @return mixed|void
     * @throws \ReflectionException
     */
    public function initializeFromJsonObject($jsonObject): void
    {
        parent::initializeFromJsonObject($jsonObject);
        if (!isset($jsonObject->categories) && !isset($jsonObject->fieldsets)) {
            throw new Exception('Category must have FieldSets or Categories!');
        }
        
        if (isset($jsonObject->categories, $jsonObject->fieldsets)) {
            throw new Exception('Category must have FieldSets or Categories but not both!');
        }
        
        if (!isset($jsonObject->id)) {
            throw new Exception('Category must have an id!');
        }
        
        if (isset($jsonObject->categories)) {
            $this->categories = new ConfigurationCategoryCollection;
            
            foreach ($jsonObject->categories as &$jsonCategoryConfig) {
                
                
                if (isset($jsonCategoryConfig->categories)
                    && $this->categoryCollectionContainsVariant($jsonCategoryConfig->categories)) {
                    
                    $jsonCategoryConfig->categories = $this->variantService->addVariantCategoriesToCategory($jsonCategoryConfig->categories);
                }
            }
            
            unset($jsonCategoryConfig);
            
            foreach ($jsonObject->categories as $jsonCategoryConfig) {
                $category = self::createFromJsonObject($jsonCategoryConfig,
                                                       $this->variantService,
                                                       $this->configurationRepository);
                $this->categories->addItem($category);
            }
        } elseif (isset($jsonObject->fieldsets)) {
            $this->fieldSets = new FieldSetCollection();
            foreach ($jsonObject->fieldsets as $fieldsetitem) {
                $this->fieldSets->addItem(FieldSet::createFromJsonObject($fieldsetitem,
                                                                         $this->configurationRepository));
            }
        }
        
        if (isset($jsonObject->for)) {
            
            $this->for = $jsonObject->for;
        }
    }
    
    
    /**
     * @param array $categories
     *
     * @return bool
     */
    protected function categoryCollectionContainsVariant(?array $categories): bool
    {
        if (!is_array($categories) || empty($categories)) {
            
            return false;
        }
        
        foreach ($categories as $category) {
            
            if (isset($category->fieldsets)) {
                
                foreach ($category->fieldsets as $fieldset) {
                    
                    if (isset($fieldset->options)) {
                        
                        foreach ($fieldset->options as $option) {
                            
                            if ($option->type === 'variant') {
                                
                                return true;
                            }
                        }
                    }
                }
            }
        }
        
        return false;
    }
    
    
    /**
     * Specify data which should be serialized to JSON
     * @link  https://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        $result = [];
        if (isset($this->categories)) {
            $result['categories'] = $this->categories();
        }
        
        if (isset($this->fieldSets)) {
            $result['fieldsets'] = $this->fieldSets();
        }
        
        $result['id'] = $this->id();
        
        if ($this->type()) {
            $result['type'] = $this->type();
        }
        
        if ($this->basic()) {
            $result['basic'] = $this->basic();
        }
        
        if ($this->title()) {
            $result['title'] = $this->title();
        }
        
        if ($this->for() !== null) {
            
            $result['for'] = $this->for();
        }
        
        return (object)$result;
    }
    
    
    /**
     * @return \Gambio\StyleEdit\Core\Components\Entities\ConfigurationCategoryCollection
     */
    public function categories(): ConfigurationCategoryCollection
    {
        return $this->categories;
    }
    
    
    /**
     * @return \Gambio\StyleEdit\Core\Components\Entities\FieldSetCollection
     */
    public function fieldSets(): ?FieldSetCollection
    {
        return $this->fieldSets;
    }
    
    
    public function __clone()
    {
        if ($this->categories) {
            $this->categories = clone $this->categories;
        }
        
        if ($this->fieldSets) {
            $this->fieldSets = clone $this->fieldSets;
        }
    }
    
    
    /**
     * @return string
     */
    public function for(): ?string
    {
        return $this->for;
    }
}
